#include "map/region.h"

#include <cstdio>
#include <fstream>
using std::ios;
using std::ofstream;
using std::ifstream;

Region::Region(Vector2<int> pos) :
    m_pos(pos) {
    // Initialize node positions
    for(int y = 0; y < REGION_SIZE; y++)
        for(int x = 0; x < REGION_SIZE; x++)
            m_nodes[y][x] = Node(pos * REGION_SIZE + Vector2<int>(x, y), {1, 1});
}

Node &Region::at(Vector2<int> pos) {
    if(pos.x() < 0 || pos.y() < 0 || pos.x() >= REGION_SIZE || pos.y() >= REGION_SIZE) {
        printf("[ERR] Region::at(%i, %i): Invalid tile location!", pos.x(), pos.y());
        return *(Node *)0;  // TODO: Crash
    }

    return m_nodes[pos.y()][pos.x()];
}

bool Region::save() {
    char path[25];
    sprintf(path, "region/%d_%d.dat", m_pos.x(), m_pos.y());
    ofstream file;
    file.open(path, ios::out | ios::binary);

    if(!file.good()) return false;

    file.write(reinterpret_cast<const char *>(&m_pos), sizeof(m_pos));

    for(int y = 0; y < REGION_SIZE; y++)
        for(int x = 0; x < REGION_SIZE; x++)
            file << m_nodes[y][x];

    file.close();
    return true;
}

bool Region::load() {
    char path[25];
    sprintf(path, "region/%d_%d.dat", m_pos.x(), m_pos.y());
    ifstream file;
    file.open(path, ios::in | ios::binary);

    if(!file.good()) return false;

    file.read(reinterpret_cast<char *>(&m_pos), sizeof(m_pos));

    for(int y = 0; y < REGION_SIZE; y++)
        for(int x = 0; x < REGION_SIZE; x++) {
            // Load node data
            file >> m_nodes[y][x];

            // Set node position
            m_nodes[y][x].set_pos(m_pos * REGION_SIZE + Vector2<int>(x, y));
        }

    file.close();
    return true;
}

