#include "util/render.h"

void Render::rectangle(Vector2<int> start, Vector2<int> end, Color color) {
    auto screen_start = m_matrix * m_view_matrix * Vector3<float>(start, 1);
    auto screen_end = m_matrix * m_view_matrix * Vector3<float>(end, 1);
    m_pge->DrawRect(
        screen_start.x(),
        screen_start.y(),
        screen_end.x() - screen_start.x(),
        screen_end.y() - screen_start.y(),
        olc::Pixel(color.r(), color.g(), color.b())
    );
}

void Render::rectangle_sz(Vector2<int> start, Vector2<int> size, Color color) {
    auto screen_start = m_matrix * m_view_matrix * Vector3<float>(start, 1);
    m_pge->DrawRect(
        screen_start.x(),
        screen_start.y(),
        size.x(),
        size.y(),
        olc::Pixel(color.r(), color.g(), color.b())
    );
}

void Render::fill_rectangle(Vector2<int> start, Vector2<int> end, Color color) {
    auto screen_start = m_matrix * m_view_matrix * Vector3<float>(start, 1);
    auto screen_end = m_matrix * m_view_matrix * Vector3<float>(end, 1);
    m_pge->FillRect(
        screen_start.x(),
        screen_start.y(),
        screen_end.x() - screen_start.x() + 1,
        screen_end.y() - screen_start.y() + 1,
        olc::Pixel(color.r(), color.g(), color.b())
    );
}

void Render::fill_rectangle_sz(Vector2<int> start, Vector2<int> size, Color color) {
    auto screen_start = m_view_matrix * Vector3<float>(start, 1);
    m_pge->FillRect(
        screen_start.x(),
        screen_start.y(),
        size.x() + 1,
        size.y() + 1,
        olc::Pixel(color.r(), color.g(), color.b())
    );
}

void Render::set_camera_pos(Vector2<int> camera_pos) {
    m_camera_pos = camera_pos;
    m_view_matrix = translation_matrix(camera_pos);
}

Vector2<int> Render::camera_pos() {
    return m_camera_pos;
}

void Render::translate(Vector2<float> trans) {
    m_matrix *= translation_matrix(trans);
}

void Render::rotate(float theta) {
    m_matrix *= rotation_matrix(theta);
}

void Render::scale(Vector2<float> scale) {
    m_matrix *= scale_matrix(scale);
}

Matrix3x3<float> Render::translation_matrix(Vector2<float> trans) {
    return Matrix3x3<float>(
        {1., 0., trans[0]},
        {0., 1., trans[1]},
        {0., 0., 1.}
    );
}

Matrix3x3<float> Render::rotation_matrix(float theta) {
    return Matrix3x3<float>(
        {cos(theta), -sin(theta), 0},
        {sin(theta), cos(theta), 0},
        {0., 0., 1.}
    );
}

Matrix3x3<float> Render::scale_matrix(Vector2<float> scale) {
    return Matrix3x3<float>(
        {scale[0], 0., 0.},
        {0., scale[1], 0.},
        {0., 0., 1.}
    );
}

