#pragma once

#include <stack>

#include "olcPixelGameEngine.h"
#include "util/color.h"
#include "util/vector2.h"
#include "util/vector3.h"
#include "util/matrix3x3.h"


class Render {
    public:
        Render(olc::PixelGameEngine *pge) : m_pge(pge) {}

        void rectangle(Vector2<int> start, Vector2<int> end, Color color = {0, 0, 0});
        void rectangle_sz(Vector2<int> start, Vector2<int> size, Color color = {0, 0, 0});

        void fill_rectangle(Vector2<int> start, Vector2<int> end, Color color = {0, 0, 0});
        void fill_rectangle_sz(Vector2<int> start, Vector2<int> size, Color color = {0, 0, 0});

        void set_camera_pos(Vector2<int> camera_pos);
        Vector2<int> camera_pos();

        void translate(Vector2<float> trans);
        void rotate(float theta);
        void scale(Vector2<float> scale);

    private:
        Matrix3x3<float> translation_matrix(Vector2<float> trans);
        Matrix3x3<float> rotation_matrix(float theta);
        Matrix3x3<float> scale_matrix(Vector2<float> scale);

    private:
        olc::PixelGameEngine *m_pge;

        Vector2<int> m_camera_pos;
        Matrix3x3<float> m_view_matrix{1};
        Matrix3x3<float> m_matrix{1};
        std::stack<Matrix3x3<float>> m_matrix_stack;
};
